@@ -627,7 +627,6 @@ accounts_dialog_setup_ui_to_add_account (EmpathyAccountsDialog *dialog)
       empathy_account_settings_set_icon_name_async (settings, "im-google-talk",
           NULL, NULL);
 
-      empathy_account_settings_set_string (settings, "server", NULL);
       empathy_account_settings_set_strv (settings, "fallback-servers",
           fallback_servers);
     }
@@ -2161,11 +2160,6 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
 
   gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 
-  /* FIXME: Remove this once we unconditionally depend on GTK+ 3 */
-#ifndef HAVE_GTK3
-  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-#endif
-
   /* add dialog buttons */
   gtk_button_box_set_layout (GTK_BUTTON_BOX (action_area), GTK_BUTTONBOX_END);
 
