@@ -30,6 +30,8 @@
 #include <glib/gi18n.h>
 
 #include <telepathy-glib/account-manager.h>
+#include <telepathy-glib/util.h>
+#include <folks/folks.h>
 
 #include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-idle.h>
@@ -40,7 +42,10 @@
 #include <libempathy/empathy-contact-list.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-gsettings.h>
+#include <libempathy/empathy-individual-manager.h>
+#include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-status-presets.h>
+#include <libempathy/empathy-tp-contact-factory.h>
 
 #include <libempathy-gtk/empathy-contact-dialogs.h>
 #include <libempathy-gtk/empathy-contact-list-store.h>
@@ -48,6 +53,9 @@
 #include <libempathy-gtk/empathy-live-search.h>
 #include <libempathy-gtk/empathy-geometry.h>
 #include <libempathy-gtk/empathy-gtk-enum-types.h>
+#include <libempathy-gtk/empathy-individual-dialogs.h>
+#include <libempathy-gtk/empathy-individual-store.h>
+#include <libempathy-gtk/empathy-individual-view.h>
 #include <libempathy-gtk/empathy-new-message-dialog.h>
 #include <libempathy-gtk/empathy-new-call-dialog.h>
 #include <libempathy-gtk/empathy-log-window.h>
@@ -83,18 +91,24 @@
 /* Name in the geometry file */
 #define GEOMETRY_NAME "main-window"
 
+enum {
+	PAGE_CONTACT_LIST = 0,
+	PAGE_NO_MATCH
+};
+
 G_DEFINE_TYPE (EmpathyMainWindow, empathy_main_window, GTK_TYPE_WINDOW);
 
-#define GET_PRIV(self) ((EmpathyMainWindowPriv*)((EmpathyMainWindow*)self)->priv)
+#define GET_PRIV(self) ((EmpathyMainWindowPriv *)((EmpathyMainWindow *) self)->priv)
 
 struct _EmpathyMainWindowPriv {
-	EmpathyContactListView  *list_view;
-	EmpathyContactListStore *list_store;
+	EmpathyIndividualStore  *individual_store;
+	EmpathyIndividualView   *individual_view;
 	TpAccountManager        *account_manager;
 	EmpathyChatroomManager  *chatroom_manager;
 	EmpathyEventManager     *event_manager;
 	guint                    flash_timeout_id;
 	gboolean                 flash_on;
+	gboolean                 empty;
 
 	GSettings              *gsettings_ui;
 	GSettings              *gsettings_contacts;
@@ -107,6 +121,8 @@ struct _EmpathyMainWindowPriv {
 	GtkWidget              *presence_chooser;
 	GtkWidget              *errors_vbox;
 	GtkWidget              *search_bar;
+	GtkWidget              *notebook;
+	GtkWidget              *no_entry_label;
 
 	GtkToggleAction        *show_protocols;
 	GtkRadioAction         *sort_by_name;
@@ -165,23 +181,24 @@ main_window_flash_foreach (GtkTreeModel *model,
 			   gpointer      user_data)
 {
 	FlashForeachData *data = (FlashForeachData *) user_data;
+	FolksIndividual *individual;
 	EmpathyContact   *contact;
 	const gchar      *icon_name;
 	GtkTreePath      *parent_path = NULL;
 	GtkTreeIter       parent_iter;
 	GdkPixbuf        *pixbuf = NULL;
 
-	/* To be used with gtk_tree_model_foreach, update the status icon
-	 * of the contact to show the event icon (on=TRUE) or the presence
-	 * (on=FALSE) */
 	gtk_tree_model_get (model, iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL,
+				&individual,
 			    -1);
 
+	if (individual == NULL)
+		return FALSE;
+
+	contact = empathy_contact_dup_from_folks_individual (individual);
 	if (contact != data->event->contact) {
-		if (contact) {
-			g_object_unref (contact);
-		}
+		tp_clear_object (&contact);
 		return FALSE;
 	}
 
@@ -189,13 +206,13 @@ main_window_flash_foreach (GtkTreeModel *model,
 		icon_name = data->event->icon_name;
 		pixbuf = empathy_pixbuf_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
 	} else {
-		pixbuf = contact_list_store_get_contact_status_icon (
-						GET_PRIV (data->window)->list_store,
-						contact);
+		pixbuf = empathy_individual_store_get_individual_status_icon (
+						GET_PRIV (data->window)->individual_store,
+						individual);
 	}
 
 	gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, pixbuf,
+			    EMPATHY_INDIVIDUAL_STORE_COL_ICON_STATUS, pixbuf,
 			    -1);
 
 	/* To make sure the parent is shown correctly, we emit
@@ -210,7 +227,8 @@ main_window_flash_foreach (GtkTreeModel *model,
 		gtk_tree_path_free (parent_path);
 	}
 
-	g_object_unref (contact);
+	g_object_unref (individual);
+	tp_clear_object (&contact);
 
 	return FALSE;
 }
@@ -226,7 +244,7 @@ main_window_flash_cb (EmpathyMainWindow *window)
 
 	priv->flash_on = !priv->flash_on;
 	data.on = priv->flash_on;
-	model = GTK_TREE_MODEL (priv->list_store);
+	model = GTK_TREE_MODEL (priv->individual_store);
 
 	events = empathy_event_manager_get_events (priv->event_manager);
 	for (l = events; l; l = l->next) {
@@ -290,7 +308,7 @@ main_window_event_removed_cb (EmpathyEventManager *manager,
 	data.on = FALSE;
 	data.event = event;
 	data.window = window;
-	gtk_tree_model_foreach (GTK_TREE_MODEL (priv->list_store),
+	gtk_tree_model_foreach (GTK_TREE_MODEL (priv->individual_store),
 				main_window_flash_foreach,
 				&data);
 }
@@ -302,19 +320,26 @@ main_window_row_activated_cb (EmpathyContactListView *view,
 			      EmpathyMainWindow      *window)
 {
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
-	EmpathyContact *contact;
+	EmpathyContact *contact = NULL;
+	FolksIndividual *individual;
 	GtkTreeModel   *model;
 	GtkTreeIter     iter;
 	GSList         *events, *l;
 
-	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->list_view));
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->individual_view));
 	gtk_tree_model_get_iter (model, &iter, path);
+
 	gtk_tree_model_get (model, &iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, &contact,
+			    EMPATHY_INDIVIDUAL_STORE_COL_INDIVIDUAL,
+				&individual,
 			    -1);
 
+	if (individual != NULL) {
+		contact = empathy_contact_dup_from_folks_individual (individual);
+	}
+
 	if (!contact) {
-		return;
+		goto OUT;
 	}
 
 	/* If the contact has an event activate it, otherwise the
@@ -335,6 +360,51 @@ main_window_row_activated_cb (EmpathyContactListView *view,
 	}
 
 	g_object_unref (contact);
+OUT:
+	tp_clear_object (&individual);
+}
+
+static void
+main_window_row_deleted_cb (GtkTreeModel      *model,
+			    GtkTreePath       *path,
+			    EmpathyMainWindow *window)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+	GtkTreeIter help_iter;
+
+	if (!gtk_tree_model_get_iter_first (model, &help_iter)) {
+		priv->empty = TRUE;
+
+		if (empathy_individual_view_is_searching (
+				priv->individual_view)) {
+			gchar *tmp;
+
+			tmp = g_strdup_printf ("<b><span size='xx-large'>%s</span></b>",
+				_("No match found"));
+
+			gtk_label_set_markup (GTK_LABEL (priv->no_entry_label), tmp);
+			g_free (tmp);
+
+			gtk_notebook_set_current_page (
+					GTK_NOTEBOOK (priv->notebook), PAGE_NO_MATCH);
+		}
+	}
+}
+
+static void
+main_window_row_inserted_cb (GtkTreeModel      *model,
+			     GtkTreePath       *path,
+			     GtkTreeIter       *iter,
+			     EmpathyMainWindow *window)
+{
+	EmpathyMainWindowPriv *priv = GET_PRIV (window);
+
+	if (priv->empty) {
+		priv->empty = FALSE;
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
+				PAGE_CONTACT_LIST);
+		gtk_widget_grab_focus (GTK_WIDGET (priv->individual_view));
+	}
 }
 
 static void
@@ -398,8 +468,7 @@ main_window_error_close_clicked_cb (GtkButton         *button,
 
 static void
 main_window_error_display (EmpathyMainWindow *window,
-			   TpAccount         *account,
-			   const gchar       *message)
+			   TpAccount         *account)
 {
 	EmpathyMainWindowPriv *priv = GET_PRIV (window);
 	GtkWidget *info_bar;
@@ -413,10 +482,19 @@ main_window_error_display (EmpathyMainWindow *window,
 	GtkWidget *action_table;
 	gchar     *str;
 	const gchar     *icon_name;
+	const gchar *error_message;
+	gboolean user_requested;
+
+	error_message =
+		empathy_account_get_error_message (account, &user_requested);
+
+	if (user_requested) {
+		return;
+	}
 
 	str = g_markup_printf_escaped ("<b>%s</b>\n%s",
 					       tp_account_get_display_name (account),
-					       message);
+					       error_message);
 
 	info_bar = g_hash_table_lookup (priv->errors, account);
 	if (info_bar) {
@@ -552,11 +630,7 @@ main_window_connection_changed_cb (TpAccount  *account,
 
 	if (current == TP_CONNECTION_STATUS_DISCONNECTED &&
 	    reason != TP_CONNECTION_STATUS_REASON_REQUESTED) {
-		const gchar *message;
-
-		message = empathy_status_reason_get_default_message (reason);
-
-		main_window_error_display (window, account, message);
+		main_window_error_display (window, account);
 	}
 
 	if (current == TP_CONNECTION_STATUS_DISCONNECTED) {
@@ -617,7 +691,7 @@ empathy_main_window_finalize (GObject *window)
 	g_list_free (priv->actions_connected);
 
 	g_object_unref (priv->account_manager);
-	g_object_unref (priv->list_store);
+	g_object_unref (priv->individual_store);
 	g_hash_table_destroy (priv->errors);
 
 	/* disconnect all handlers of status-changed signal */
@@ -693,7 +767,7 @@ static void
 main_window_chat_add_contact_cb (GtkAction         *action,
 				 EmpathyMainWindow *window)
 {
-	empathy_new_contact_dialog_show (GTK_WINDOW (window));
+	empathy_new_individual_dialog_show (GTK_WINDOW (window));
 }
 
 static void
@@ -715,10 +789,8 @@ main_window_view_show_offline_cb (GtkToggleAction   *action,
 				EMPATHY_PREFS_UI_SHOW_OFFLINE,
 				current);
 
-	/* Turn off sound just while we alter the contact list. */
-	// FIXME: empathy_sound_set_enabled (FALSE);
-	empathy_contact_list_store_set_show_offline (priv->list_store, current);
-	//empathy_sound_set_enabled (TRUE);
+	empathy_individual_view_set_show_offline (priv->individual_view,
+			current);
 }
 
 static void
@@ -736,7 +808,7 @@ main_window_notify_sort_contact_cb (GSettings         *gsettings,
 		GEnumClass *enum_class;
 		GEnumValue *enum_value;
 
-		type = empathy_contact_list_store_sort_get_type ();
+		type = empathy_individual_store_sort_get_type ();
 		enum_class = G_ENUM_CLASS (g_type_class_peek (type));
 		enum_value = g_enum_get_value_by_nick (enum_class, str);
 		if (enum_value) {
@@ -768,7 +840,7 @@ main_window_view_sort_contacts_cb (GtkRadioAction    *action,
 	group = gtk_radio_action_get_group (action);
 
 	/* Get string from index */
-	type = empathy_contact_list_store_sort_get_type ();
+	type = empathy_individual_store_sort_get_type ();
 	enum_class = G_ENUM_CLASS (g_type_class_peek (type));
 	enum_value = g_enum_get_value (enum_class, g_slist_index (group, current));
 
@@ -780,7 +852,8 @@ main_window_view_sort_contacts_cb (GtkRadioAction    *action,
 				       EMPATHY_PREFS_CONTACTS_SORT_CRITERIUM,
 				       enum_value->value_nick);
 	}
-	empathy_contact_list_store_set_sort_criterium (priv->list_store, value);
+	empathy_individual_store_set_sort_criterium (priv->individual_store,
+			value);
 }
 
 static void
@@ -795,8 +868,8 @@ main_window_view_show_protocols_cb (GtkToggleAction   *action,
 	g_settings_set_boolean (priv->gsettings_ui,
 				EMPATHY_PREFS_UI_SHOW_PROTOCOLS,
 				value);
-	empathy_contact_list_store_set_show_protocols (priv->list_store,
-						       value);
+	empathy_individual_store_set_show_protocols (priv->individual_store,
+						     value);
 }
 
 /* Matches GtkRadioAction values set in empathy-main-window.ui */
@@ -830,10 +903,11 @@ main_window_view_contacts_list_size_cb (GtkRadioAction    *action,
 				value == CONTACT_LIST_COMPACT_SIZE);
 	g_settings_apply (gsettings_ui);
 
-	empathy_contact_list_store_set_show_avatars (priv->list_store,
-						     value == CONTACT_LIST_NORMAL_SIZE_WITH_AVATARS);
-	empathy_contact_list_store_set_is_compact (priv->list_store,
-						   value == CONTACT_LIST_COMPACT_SIZE);
+	/* FIXME: these enums probably have the wrong namespace */
+	empathy_individual_store_set_show_avatars (priv->individual_store,
+			value == CONTACT_LIST_NORMAL_SIZE_WITH_AVATARS);
+	empathy_individual_store_set_is_compact (priv->individual_store,
+			value == CONTACT_LIST_COMPACT_SIZE);
 
 	g_object_unref (gsettings_ui);
 }
@@ -880,7 +954,7 @@ static void
 main_window_view_show_map_cb (GtkCheckMenuItem  *item,
 			      EmpathyMainWindow *window)
 {
-#if HAVE_LIBCHAMPLAIN
+#ifdef HAVE_LIBCHAMPLAIN
 	empathy_map_view_show ();
 #endif
 }
@@ -890,16 +964,13 @@ join_chatroom (EmpathyChatroom *chatroom,
 	       gint64 timestamp)
 {
 	TpAccount      *account;
-	TpConnection   *connection;
 	const gchar    *room;
 
 	account = empathy_chatroom_get_account (chatroom);
-	connection = tp_account_get_connection (account);
-	g_assert (connection != NULL);
 	room = empathy_chatroom_get_room (chatroom);
 
 	DEBUG ("Requesting channel for '%s'", room);
-	empathy_dispatcher_join_muc (connection, room, timestamp, NULL, NULL);
+	empathy_dispatcher_join_muc (account, room, timestamp);
 }
 
 typedef struct
@@ -957,6 +1028,9 @@ account_status_changed_cb (TpAccount  *account,
 		case TP_CONNECTION_STATUS_CONNECTED:
 			/* We can join the room */
 			break;
+
+		default:
+			g_assert_not_reached ();
 	}
 
 	join_chatroom (ctx->chatroom, ctx->timestamp);
@@ -1144,7 +1218,8 @@ main_window_edit_cb (GtkAction         *action,
 	GtkWidget *submenu;
 
 	/* FIXME: It should use the UIManager to merge the contact/group submenu */
-	submenu = empathy_contact_list_view_get_contact_menu (priv->list_view);
+	submenu = empathy_individual_view_get_individual_menu (
+			priv->individual_view);
 	if (submenu) {
 		GtkMenuItem *item;
 		GtkWidget   *label;
@@ -1161,7 +1236,8 @@ main_window_edit_cb (GtkAction         *action,
 		return;
 	}
 
-	submenu = empathy_contact_list_view_get_group_menu (priv->list_view);
+	submenu = empathy_individual_view_get_group_menu (
+			priv->individual_view);
 	if (submenu) {
 		GtkMenuItem *item;
 		GtkWidget   *label;
@@ -1458,6 +1534,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 {
 	EmpathyMainWindowPriv    *priv;
 	EmpathyContactList       *list_iface;
+	EmpathyIndividualManager *individual_manager;
 	GtkBuilder               *gui;
 	GtkWidget                *sw;
 	GtkToggleAction          *show_offline_widget;
@@ -1466,6 +1543,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	gboolean                  show_offline;
 	gchar                    *filename;
 	GSList                   *l;
+	GtkTreeModel             *model;
 
 	priv = window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window,
 			EMPATHY_TYPE_MAIN_WINDOW, EmpathyMainWindowPriv);
@@ -1494,6 +1572,8 @@ empathy_main_window_init (EmpathyMainWindow *window)
 				       "view_show_map", &show_map_widget,
 				       "room_join_favorites", &priv->room_join_favorites,
 				       "presence_toolbar", &priv->presence_toolbar,
+				       "notebook", &priv->notebook,
+				       "no_entry_label", &priv->no_entry_label,
 				       "roster_scrolledwindow", &sw,
 				       NULL);
 	g_free (filename);
@@ -1534,7 +1614,7 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	g_object_ref (priv->ui_manager);
 	g_object_unref (gui);
 
-#if !HAVE_LIBCHAMPLAIN
+#ifndef HAVE_LIBCHAMPLAIN
 	gtk_action_set_visible (show_map_widget, FALSE);
 #endif
 
@@ -1595,33 +1675,51 @@ empathy_main_window_init (EmpathyMainWindow *window)
 	priv->throbber_tool_item = GTK_WIDGET (item);
 
 	list_iface = EMPATHY_CONTACT_LIST (empathy_contact_manager_dup_singleton ());
-	priv->list_store = empathy_contact_list_store_new (list_iface);
-	priv->list_view = empathy_contact_list_view_new (priv->list_store,
-							 EMPATHY_CONTACT_LIST_FEATURE_ALL,
-							 EMPATHY_CONTACT_FEATURE_ALL);
+	individual_manager = empathy_individual_manager_dup_singleton ();
+	priv->individual_store = empathy_individual_store_new (
+			individual_manager);
+	g_object_unref (individual_manager);
+
+	/* For the moment, we disallow Persona drops onto the main contact list (e.g. from things such as
+	 * the EmpathyPersonaView in the linking dialogue). No code is hooked up to do anything on a Persona
+	 * drop, so allowing them would achieve nothing except confusion. */
+	priv->individual_view = empathy_individual_view_new (
+			priv->individual_store,
+			EMPATHY_INDIVIDUAL_VIEW_FEATURE_ALL ^ EMPATHY_INDIVIDUAL_VIEW_FEATURE_PERSONA_DROP,
+			EMPATHY_INDIVIDUAL_FEATURE_ALL);
 
 	priv->butterfly_log_migration_members_changed_id = g_signal_connect (
-		list_iface, "members-changed",
-		G_CALLBACK (main_window_members_changed_cb), window);
+			list_iface, "members-changed",
+			G_CALLBACK (main_window_members_changed_cb), window);
 
 	g_object_unref (list_iface);
 
-	gtk_widget_show (GTK_WIDGET (priv->list_view));
+	gtk_widget_show (GTK_WIDGET (priv->individual_view));
 	gtk_container_add (GTK_CONTAINER (sw),
-			   GTK_WIDGET (priv->list_view));
-	g_signal_connect (priv->list_view, "row-activated",
+			   GTK_WIDGET (priv->individual_view));
+	g_signal_connect (priv->individual_view, "row-activated",
 			  G_CALLBACK (main_window_row_activated_cb),
 			  window);
 
 	/* Set up search bar */
 	priv->search_bar = empathy_live_search_new (
-		GTK_WIDGET (priv->list_view));
-	empathy_contact_list_view_set_live_search (priv->list_view,
+		GTK_WIDGET (priv->individual_view));
+	empathy_individual_view_set_live_search (priv->individual_view,
 		EMPATHY_LIVE_SEARCH (priv->search_bar));
 	gtk_box_pack_start (GTK_BOX (priv->main_vbox), priv->search_bar,
 		FALSE, TRUE, 0);
 	g_signal_connect_swapped (window, "map",
-		G_CALLBACK (gtk_widget_grab_focus), priv->list_view);
+		G_CALLBACK (gtk_widget_grab_focus), priv->individual_view);
+
+	/* Connect to proper signals to check if contact list is empty or not */
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->individual_view));
+	priv->empty = TRUE;
+	g_signal_connect (model, "row-inserted",
+			  G_CALLBACK (main_window_row_inserted_cb),
+			  window);
+	g_signal_connect (model, "row-deleted",
+			  G_CALLBACK (main_window_row_deleted_cb),
+			  window);
 
 	/* Load user-defined accelerators. */
 	main_window_accels_load ();
@@ -1631,13 +1729,11 @@ empathy_main_window_init (EmpathyMainWindow *window)
 
 	/* Enable event handling */
 	priv->event_manager = empathy_event_manager_dup_singleton ();
-	g_signal_connect (priv->event_manager, "event-added",
-			  G_CALLBACK (main_window_event_added_cb),
-			  window);
-	g_signal_connect (priv->event_manager, "event-removed",
-			  G_CALLBACK (main_window_event_removed_cb),
-			  window);
 
+	g_signal_connect (priv->event_manager, "event-added",
+			  G_CALLBACK (main_window_event_added_cb), window);
+	g_signal_connect (priv->event_manager, "event-removed",
+			  G_CALLBACK (main_window_event_removed_cb), window);
 	g_signal_connect (priv->account_manager, "account-validity-changed",
 			  G_CALLBACK (main_window_account_validity_changed_cb),
 			  window);
@@ -1650,8 +1746,8 @@ empathy_main_window_init (EmpathyMainWindow *window)
 
 	l = empathy_event_manager_get_events (priv->event_manager);
 	while (l) {
-		main_window_event_added_cb (priv->event_manager,
-					    l->data, window);
+		main_window_event_added_cb (priv->event_manager, l->data,
+				window);
 		l = l->next;
 	}
 
